@@ -501,6 +501,8 @@
          (c-forward-conditional 1)
          (beginning-of-line 0)
          :ifdef-comment)
+        ((looking-at "#ifdef")
+         :ifdef)
         ((looking-at "#ifndef")
          :ifndef)
         ((looking-at "#if [^0]")
@@ -516,6 +518,8 @@
          :c-comment)
         ((looking-at "#import")
          :import)
+        ((looking-at "#include")
+         :include)
         ((looking-at "")
          :page-break)
         ((looking-at "typedef\\s-+")
@@ -529,7 +533,7 @@
          :macro)
         ((looking-at "@class")
          :class)
-        ((looking-at "extern\\s-+\\(Class\\s-+\\|int\\s-+\\|void\\s-\\|id\\s-+<.*\\s-\\|id\\s-\\|const\\s-+char\\s-*\\*\\)\\s-*\\([^ (]+\\)\\s-*(")
+        ((looking-at "extern\\s-+\\(Class\\s-+\\|int\\s-+\\|void\\s-\\|id\\s-+<.*\\s-\\|id\\s-\\|const\\s-+char\\s-*\\*\\|BOOL\\)\\s-*\\([^ (]+\\)\\s-*(")
          (let ((return-type (match-string 1))
                (function-name (match-string 2)))
            (setf (parse-state-function-return-type parse-state)
@@ -1213,14 +1217,17 @@
           (insert "\n"))
     (insert "</FUNCSYNOPSISINFO>\n"))
 
-(defun print-method-signature (method &optional stream)
+(defun print-method-signature (method &optional stream nocolon)
   (if (method-factory-flag method)
       (princ "+" stream)
       (princ "-" stream))
   (loop for arguments in (method-arguments method)
         for key = (first arguments)
         when key do (princ key stream)
-        when (third arguments) do (princ ":" stream)))
+        when (third arguments) do 
+        (unless nocolon
+            (princ ":" stream))
+        ))
 
 (defun sgml-method-funcsynopsis (owner-protocol method)
   (insert "<FUNCSYNOPSIS ID=\"")
@@ -1483,7 +1490,17 @@
       (insert "/")
       (insert (external-protocol-name protocol))
       (insert (format "/%d" (general-example-counter protocol)))
+
+      ; start ID tag attribute                                        
+      (insert "\" ID=\"SWARM.")
+      (insert (module-name (protocol-module protocol)))
+      (insert ".")
+      (insert (external-protocol-name protocol))
+      (insert ".GENERIC")
+      (insert (format ".%d" (general-example-counter protocol)))
+      (insert ".EXAMPLE")
       (insert "\">")
+
       (insert "<TITLE>\n")
       (insert "</TITLE>\n")
       (loop for example in example-list
@@ -1523,17 +1540,26 @@
     (insert "/")
     (print-method-signature method (current-buffer))
     (insert (format "/%d" (method-example-counter protocol method)))
+
+    ; start ID tag attribute                                        
+    (insert "\" ID=\"SWARM.")
+    (insert (module-name (protocol-module protocol)))
+    (insert ".")
+    (insert (external-protocol-name protocol))
+    (insert ".")
+    (print-method-signature method (current-buffer) t)
+    (insert (format ".%d" (method-example-counter protocol method)))
+    (insert ".EXAMPLE")
+
     (insert "\">")
     (insert "<TITLE>")
     (insert "</TITLE>\n")
 
-    (insert "<PROGRAMLISTING>\n")
-
     (loop for example in (method-example-list method)
           do
+          (insert "<PROGRAMLISTING>\n<![ CDATA [\n")
           (insert example)
-          (insert "\n"))
-    (insert "</PROGRAMLISTING>\n")
+          (insert "]]>\n</PROGRAMLISTING>\n"))
     (insert "</EXAMPLE>\n")))
 
 (defun sgml-methods-for-phase (protocol phase)
@@ -1752,8 +1778,161 @@
 (defun run-all ()
   (interactive)
   (load-and-process-modules)
-  (sgml-create-refentries-for-all-modules)
-  (sgml-generate-indices)
+  (sgml-create-refentries-for-all-modules)  (sgml-generate-indices)
   nil)
 
+(defun vcg-graph-all-protocols ()
+  (loop for module-sym being each hash-key of *module-hash-table*
+        using (hash-values module-items)
+        for module-name = (concat "M:" (symbol-name module-sym))
+        do
+        (insert "graph: {\nfolding: 1 color: blue title: \"")
+        (insert (capitalize module-name))
+        (insert "\"\n")
+        (insert "node: { title: \"")
+        (insert module-name)
+        (insert "\" }\n")
+        (loop for module-item in module-items do
+              (cond ((and (protocol-p module-item)
+                          (not (internal-protocol-p module-item)))
+                     (let ((protocol-name (protocol-name module-item)))
+                       (insert "node: { title: \"")
+                       (insert protocol-name)
+                       (insert "\" shape: ellipse color: red }\n")
+                       (insert "edge: { targetname: \"")
+                       (insert protocol-name)
+                       (insert "\" sourcename: \"")
+                       (insert module-name)
+                       (insert "\" }\n"))
+                     (loop for included-protocol in
+                           (protocol-included-protocol-list module-item)
+                           unless (internal-protocol-p included-protocol)
+                           do
+                           (insert "backedge: { targetname: \"")
+                           (insert (protocol-name module-item))
+                           (insert "\" sourcename: \"")
+                           (insert (protocol-name included-protocol))
+                           (insert "\" }\n")))))
+        (insert "}\n")))
+
+(defun vcg-output-protocols-graph ()
+  (interactive)
+  (with-temp-file (concat (get-swarmdocs-build-area) "protocols.vcg")
+    (insert "graph: {\n")
+    (insert "orientation: left_to_right\n")
+    (vcg-graph-all-protocols)
+    (insert "}\n")))
+
+(defun dot-graph-all-protocols ()
+  (loop for module-sym being each hash-key of *module-hash-table*
+        using (hash-values module-items)
+        for module-name = (symbol-name module-sym)
+        do
+        (loop for module-item in module-items do
+              (cond ((and (protocol-p module-item)
+                          (not (internal-protocol-p module-item)))
+                     (loop for included-protocol in
+                           (protocol-included-protocol-list module-item)
+                           unless (internal-protocol-p included-protocol)
+                           do
+                           (insert "\"")
+                           (insert (protocol-name module-item))
+                           (insert "\" -> \"")
+                           (insert (protocol-name included-protocol))
+                           (insert "\"\n")))))
+        (insert "subgraph cluster_")
+        (insert module-name)
+        (insert " { label=\"")
+        (insert (capitalize module-name))
+        (insert "\"\n")
+        (loop for module-item in module-items do
+              (cond ((and (protocol-p module-item)
+                          (not (internal-protocol-p module-item)))
+                     (insert "\"")
+                     (insert (protocol-name module-item))
+                     (insert "\"; "))))
+        (insert "}\n")))
+
+(defun dot-output-protocols-graph ()
+  (interactive)
+  (with-temp-file (concat (get-swarmdocs-build-area) "protocols.dot")
+    (insert "digraph \"Swarm Protocols\" {\n");
+    (insert "page=\"10,7.5\"\n")
+    (insert "ratio=auto\n")
+    (dot-graph-all-protocols)
+    (insert "}\n")))
+
+(defun dot-graph-module (edge-hash-table module-sym &optional module-items)
+  (unless module-items 
+    (setq module-items (gethash module-sym *module-hash-table*)))
+  (let ((module-name (symbol-name module-sym))
+        (included-module-hash-table (make-hash-table))
+        local-protocols)
+    (loop for module-item in module-items do
+          (cond ((and (protocol-p module-item)
+                      (not (internal-protocol-p module-item)))
+                 (loop for included-protocol in
+                       (protocol-included-protocol-list module-item)
+                       for included-module-sym = 
+                       (module-sym (protocol-module included-protocol))
+                       unless (internal-protocol-p included-protocol)
+                       do
+                       (let ((edge-key (cons module-item included-protocol)))
+                         (unless (gethash edge-key edge-hash-table)
+                           (insert "\"")
+                           (insert (protocol-name module-item))
+                           (insert "\" -> \"")
+                           (insert (protocol-name included-protocol))
+                           (insert "\"\n")
+                           (setf (gethash edge-key edge-hash-table) t)))
+                       (if (eq included-module-sym module-sym)
+                           (push included-protocol local-protocols)
+                           (push included-protocol
+                                 (gethash included-module-sym
+                                          included-module-hash-table))
+                           )))))
+    (insert "subgraph cluster_")
+    (insert module-name)
+    (insert " { label=\"")
+    (insert (capitalize module-name))
+    (insert "\"\n")
+    (loop for module-item in (append module-items local-protocols) do
+          (cond ((and (protocol-p module-item)
+                      (not (internal-protocol-p module-item)))
+                 (insert "\"")
+                 (insert (protocol-name module-item))
+                 (insert "\"; "))))
+    (insert "}\n")
+    (loop for included-module-sym being each hash-key of
+          included-module-hash-table
+          do
+          (dot-graph-module edge-hash-table
+                            included-module-sym 
+                            (gethash included-module-sym
+                                     included-module-hash-table)))))
+
+(defun dot-output-module-graph (module-sym &optional compress-flag)
+  (let ((edge-hash-table (make-hash-table :test #'equal)))
+    (with-temp-file (concat (get-swarmdocs-build-area)
+                            (symbol-name module-sym)
+                            (if compress-flag "-singlepage" "")
+                            ".dot")
+      (insert "digraph ")
+      (insert (capitalize (symbol-name module-sym)))
+      (insert " {\n")
+      (if compress-flag
+          (progn
+            (insert "size=\"10,7.5\"\n")
+            (insert "ratio=compress\n")
+            (insert "rotate=90\n"))
+          (progn
+            (insert "ratio=auto\n")
+            (insert "page=\"7.5,10\"\n")))
+      (dot-graph-module edge-hash-table module-sym)
+      (insert "}\n"))))
+
+(defun dot-output-each-module-graph (&optional compress-flag)
+  (interactive "P")
+  (loop for module-sym being each hash-key of *module-hash-table* do
+        (dot-output-module-graph module-sym compress-flag)))
 
